;;; Copyright (C) 1992, Digital Equipment Corporation
;;; All rights reserved.
;;; See the file COPYRIGHT for a full description.
;;;
;;; Created by meehan on Sun Aug 16 15:27:29 PDT 1992
;;; Last modified on Sun Aug 16 15:49:38 PDT 1992 by meehan

#| This program reads the lines in the file "color.list", which is
   assumed to be in alphabetical order, and writes a LaTex fragment
   in which the names are arranged as a table in column-major order,
   as in the output from "ls(1)".
   
   "color.list" was produced by hand from the names in
   /proj/m3/color/src/ColorNameTable.i3 (which was generated by
   /proj/m3/color/tools/src/MakeBasicTable.m3).
   
   The LaTex should be placed in /proj/m3/color/src/ColorName.i3,
   where it will become part of the VBTkit manual.
|#

(in-package "USER")

(defun f (width)
  ;; Print the names from "color.list" as a table with "width" columns
  ;; and as many rows as needed. If the number of items in the list
  ;; is not an exact multiple of "width", the last N rows will have
  ;; "width" - 1 elements. The output goes to *standard-output*. 
  (let ((names nil))
    (with-open-file (in "color.list")
      (loop (let ((name (read-line in nil)))
               (if name (push name names) (return)))))
    (setq names (nreverse names))
    (format t "%  There are ~d name~:P.~%" (length names))
    (let* ((n (length names))
           (height (ceiling n width))
           (v (make-array (list height width))))
      (dotimes (col width)
        (dotimes (row height)
          (setf (aref v row col) (pop names))))
      (format t "    \\begin{tt}~%    "\\begin{tabular}{")
      (dotimes (i width) (princ "l"))
      (format t "}~%    )
      (dotimes (row height)
        (dotimes (col width)
          (let ((x (aref v row col)))
            (cond (x (if (> col 0) (princ " & "))
                     (princ x)))))
        (format t " \\\\~%    "))
      (format t "\\end{tabular}~%    \\end{tt}~%")
    )))

(defun to-file (file &optional (width 4))
  ;; Re-direct *standard-output* to "file", and call (f width).
  (with-open-file (*standard-output* file
                      :direction :output
                      :if-exists :supersede)
    (f width)))
